/*
$Id: create.c,v 1.11 1995/10/17 02:26:48 ssmore Exp $

File Name : create.c
Author    : ssmore

Entry Point(s) :

(1) PASSION_create   prototype in passion.h

Modification History :

Module Description                                      Date      Author
------------------------------------------------------------------------------
(1) PASSION_create                                        2/22/95    ssmore
(2) PASSION_create                                        8/24/95    ssmore

Notes :

(1) Modifications to the entry points interface should be reflected in the 
    prototypes declarations.
*/

#ifndef PASSION_CREATE_C
#define PASSION_CREATE_C

#if HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#ifdef PARAGON
#include <nx.h>
#endif

#include <fcntl.h>

#include "pfile.h"
/*
#include "ocad.h"
#include "passion.h"
*/

#define DEBUG

#ifdef DEBUG
#include <stdio.h>
#endif

#ifndef WIFEXITED
# define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
#endif

#ifndef WEXITSTATUS
# define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
#endif

/*
   written by Zheng Gengbin

   if reuse_fd set to 1,
	File Handler Usage is Enabled. 
	I manage a STACK to preserve every fd created for LAFs. 
	When new LAF is needed, stack will be search for an already opened one,
	and LAF_close() donot really close LAFs but push the LAF information to
	stack.
*/
int reuse_fd = 1;

PSTACK stack_ptr=NULL;

void push(LAF_File fd, char *filename)
{
   PSTACK newptr;
   newptr = (PSTACK) malloc(sizeof(STACK));
   newptr->fd = fd;
   newptr->filename = filename;		/* saved for remove */
   newptr->next = NULL;
   if (stack_ptr == NULL)
	stack_ptr = newptr;
   else {
	newptr->next = stack_ptr;
	stack_ptr = newptr;	
   }
}

PSTACK pop()
{
   PSTACK ptr;
   LAF_File fd;
   if (stack_ptr == NULL) return NULL;
   ptr = stack_ptr;
   stack_ptr = stack_ptr->next;
   return ptr;
}

int empty()
{
   return stack_ptr==NULL?1:0;
}


/*
  Name : PASSION_create

  Function : This function is used to create the Local Array File (LAF)

  Parameters : 
    Input :
      FileName - pointer to a ASCIIZ string, it is the name of the LAF
		 to be created
    Output :
      none
    Input/Ooutput :
      none

  Return Value : 
    Pointer to PFILE structure, if successful
    (PFILE *)0, if not successful

  Algorithm :

    if file name is not specified
      return error 
    endif

    allocate space for the parallel file data structure (PFILE)
    if out of memory space
      return error
    endif

		create the file
		allocate space for the file
    open the file for read/write
    if not successful
      return error
    endif

    put the file descriptor in the PFILE structure

    return the pointer to the PFILE structure

  Side Effects :
    (1) Allocates a PFILE structure that has to be de-allocated when the file is
        closed.

  Shortcomings :
    (1) Cannot create global file.

  Modifications :

  Description                                       Date          Author
  ------------------------------------------------------------------------------
	(1) Since overlap area is no more stored          8/24/95       ssmore
			in LAF the file size calculations changed
*/
PFILE *PASSION_create(char *FileName, unsigned int HeaderSize, long int filesize)
{
PFILE *PFilePtr;  /* pointer to the PFILE buffer, is the return value, if all 
		     goes well */
LAF_File fd;
long int i;

  if (FileName == (char *)0) 
	{
#ifdef DEBUG
    fprintf(stderr,"PASSION : (create) No filename specified\n");
#endif
		return (PFILE *)0;
  }

  if (FileName[0] == (char)0) 
	{
#ifdef DEBUG
    fprintf(stderr,"PASSION : (create) No filename specified\n");
#endif
		return (PFILE *)0;
  }


	/*
	File is created with rw-r--r-- permissions
	*/
#ifdef HAVE_MPIO
  {
  char prefix[500];
  strcpy(prefix,"ufs:");
  strcpy(prefix+4, FileName);
  MPI_File_open(MPI_COMM_SELF, prefix, MPI_MODE_RDWR | MPI_MODE_CREATE, MPI_INFO_NULL, &fd);
#ifndef DEBUG
  MPI_File_set_size(fd, filesize);
#endif
  MPI_File_close(&fd); 
  }
#else
      /*  fd = open(FileName, O_RDWR | O_CREAT | O_TRUNC, 0644); */
  fd = open(FileName, O_RDWR | O_CREAT , 0644); 
  if (fd == -1)
  {
#ifdef DEBUG
    fprintf(stderr,"PASSION : (create) System Error\n");
#endif
#ifdef DEBUG
    perror("create");
#endif
    return (PFILE *)0;
  }

#ifdef PARAGON
  if(lsize(fd, (off_t)filesize, SIZE_SET) != (long)filesize)
	{
#ifdef DEBUG
    fprintf(stderr,"PASSION : (create) System Error\n");
#endif
#ifdef DEBUG
    perror("lsize");
#endif
    close(fd);
    return (PFILE *)0;
	}
	close(fd);
#else
	/* allocate LAF space */
	/* lseek can set beyond the end of the file, so nothing is done here. */
	/* shell scripts */
#endif
#endif

	PFilePtr = (PFILE *)PASSION_open((char *)FileName, (unsigned int)HeaderSize);
	PFilePtr->share->Status = PASSION_NEW; 

  return PFilePtr;
}

/* Zheng Gengbin */
PFILE *LAF_create(int proc_id,unsigned int HeaderSize, long int filesize)
{
  PFILE *PFilePtr;
  SHARE_PFILE *ShPFilePtr;
  PSTACK  sptr;
  char *FileName;
  LAF_File fd;

  if (reuse_fd && !empty()) {
    sptr = pop();
    fd = sptr->fd;
    FileName = sptr->filename;
    free(sptr);
    PFilePtr = (PFILE *)malloc(sizeof(PFILE));
    ShPFilePtr = (SHARE_PFILE *)malloc(sizeof(SHARE_PFILE));
    if (PFilePtr == (PFILE *)0 || ShPFilePtr == (SHARE_PFILE *)0)
    {
#ifdef DEBUG
    fprintf(stderr,"PASSION : (create) Not enough memory\n");
#endif
    return (PFILE *)0;
    }

    ShPFilePtr->filename = FileName;
    ShPFilePtr->Fd = fd;
    ShPFilePtr->Status = PASSION_NEW;
    PFilePtr->share = ShPFilePtr;
    PFilePtr->HeaderSize = HeaderSize;
      /* reset file pointer */
/*
#ifdef HAVE_MPIO
    MPI_File_set_size(ShPFilePtr->Fd, 0);
#endif
    if (lseek(fd, (off_t) 0, SEEK_SET) == (off_t)-1)
    {
#ifdef DEBUG
      fprintf(stderr,"PASSION : (create) error\n");
#endif
    }
*/
    return PFilePtr;
  }
  else {
   char name[1024];
   sprintf(name,"%s%d-%d","LAF",proc_id);
   FileName=tempnam(".",name);
   return PASSION_create(FileName, HeaderSize, filesize);
  }
}

#endif  /* PASSION_CREATE_C */
